%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                  Step_length_update.m
%                   Russell Luke
%  Department of MathSci, University of Delaware  
%                        and 
%  Inst. f. Num. u. Angew. Math., Universitaet Goettingen
%                    JUNE. 2, 2010  
%
%  This work was supported by NSF-DMS grant 0712796. 
%
% DESCRIPTION:
%   Trust region adjustment routine
%
% INPUT:
%
% OUTPUT:
%
% USAGE:
% 
% SUBROUTINE CALLS:
%      Hessian_product: invMSB2_product, invBFGS2_product, BFGS1_product,
%                       and MSB1_product
%

function [tr,mu_vec,cDDf, lower_step_bound,d_vec, stepsize_vec, accept]=...
    Step_length_update(History, Hessian_product, update_conditions,...
    mem,maxmem,iter,dim,...
    x_mat, f_vec,gradf_mat,d_vec,mu_vec,stepsize_vec,...
    tr,delta, alpha, eta1, eta2, eta3,cDDf,DDfnew, bDDf, orthog_TOL, lower_step_bound, gamma)

if(strcmp(update_conditions,'Exact'))
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % exact line search via bisection
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if(mu_vec(iter)==0)
        tr=1e+15;
        lower_step_bound=0;
        mu_vec(iter)=1;
    end
    min_d=lower_step_bound;
    max_d=tr;
    nd=stepsize_vec(iter);
    % disp('in Exact'), keyboard
    if((abs(DDfnew)/stepsize_vec(iter)<=orthog_TOL))
        accept=1;
    else
        accept=-2;
        if((f_vec(iter+1) <= f_vec(iter))&&(DDfnew/nd<-orthog_TOL))
            % step too short:
            if abs(nd-max_d)/(max_d-min_d)<1e-2
                max_d=1e+7*max_d;
                tr=max_d;
                d_vec=2*d_vec;
            else
                % increase step length by one half the interval to the outer bound
                d_vec=(min((nd+max_d)/2,10*nd))*d_vec/nd;
            end
            min_d=nd;
        elseif((f_vec(iter+1) <= f_vec(iter))&&(DDfnew/nd>orthog_TOL))
            % step too long, but not by too much:  decrease step length by one half the
            %                                      interval to the inner bound
            d_vec=(nd+min_d)/2*(d_vec/nd);
            max_d=nd;
            tr=max_d;
        elseif((f_vec(iter+1) > f_vec(iter)))
            % step too long, by a lot:  decrease step length by one half the current steplength
            max_d=nd;
            tr=max_d;
            d_vec=0.5*(min_d+nd)*(d_vec/nd);
        end
    end
    lower_step_bound = min_d;
    stepsize_vec(iter)  = feval('norm',d_vec);

else % do backtracking linesearch/ adjust the trust region
    if(mem>0)
        S_mat         = zeros(dim,mem);
        Y_mat         = zeros(dim,mem);
        [S_mat,Y_mat] = feval(History,dim,mem+1,x_mat(:,maxmem-mem:maxmem),gradf_mat(:,maxmem-mem:maxmem));
    end
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Adjust trust region:
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % actual change/predicted change
    p=max(mem-1,1);  % look-back - this is a type of nonmonotone trust region adjustment
    pind = max(iter-p,1);
    pred = 0.;
    if(mem>0)
        pred = (gradf_mat(:,maxmem)'*(d_vec)+0.5*d_vec'*feval(Hessian_product,dim,mem,d_vec,S_mat,Y_mat,delta, alpha));
        actual = f_vec(iter+1)-f_vec(iter);
        mu_vec(iter) = (actual)/pred;
        if(abs(pred-actual)/abs(pred)>1e+5)
            display('Warning: step length may be too small -- samsara results might be at numerical accuracy.')
            display('Try increasing your step length tolerance.')
        end
    else
        mu_vec(iter) = (f_vec(iter+1)-f_vec(iter))/(gradf_mat(:,maxmem)'*(d_vec));
    end
    %    disp('in TR_adjust')
    %    keyboard
    % save('mat_vars.mat', 'mem', 'mu_vec', 'pred', 'stepsize_vec', ...
    %      'tr', 'pind')
    if(mem>0)
        if (mu_vec(iter)<=0||pred>0)
            tr  = min(.1*stepsize_vec(iter-1), .1*tr); 
            accept=-1;
        elseif (sum(mu_vec(pind+1:iter))/(iter-pind)>=eta1)
            tr    = min([5*tr,1e+15]);
            accept=1;
        elseif (sum(mu_vec(pind+1:iter))/(iter-pind)>=eta2)
            tr    = min([2*tr, 1e+15]);
            accept=1;
        elseif (sum(mu_vec(pind+1:iter))/(iter-pind)>eta3)
            tr=1.2*tr; % Nocedal & Wright would suggest leaving tr the same
            accept=1;
        else
            tr    = min(stepsize_vec(iter), .25*tr);
            if ((f_vec(iter+1) <= max(f_vec(max(1,iter-p):iter))+cDDf)&&(DDfnew>=bDDf))
                accept=1;
            elseif(f_vec(iter+1) <= max(f_vec(max(1,iter-p):iter)))&& (stepsize_vec(iter)<2*stepsize_vec(iter-1))
                accept=0; % still got decrease, so will keep the information, but not advance the point
            else
                accept=-1; % no decrease and stepsize might be too small, shrink memory and don't keep info
            end
        end
    else
        max_d=tr;
        nd=stepsize_vec(iter);
        if(lower_step_bound>=max_d)
            min_d=0;
        else
            min_d=lower_step_bound;
        end
        % weakness of this method.  In general don't know what max_d
        % should be, and if you underestimate the routine will stagnate.
        if (abs(nd-max_d)/(max_d-min_d)<1e-2)
            max_d=1e+7*max_d;
            tr=max_d;
        end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % nonmonotone bisection line search
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % DDfnew/nd
        p=max(mem,1);
        if(((f_vec(iter+1) <= max(f_vec(max(1,iter-p):iter))+cDDf)&&(DDfnew>=bDDf)))
            % Wolfe conditions satisfied, do not change step.
            accept=1;
        else
            accept=-2;
            if((f_vec(iter+1) <= max(f_vec(max(1,iter-p):iter))+cDDf)&&(DDfnew<bDDf))
                if((max_d-min_d)/max_d<=1e-2) % probably underestimating the upper bound
                    max_d=10*max_d;
                    tr=max_d;
                end
                % step too short:
                % increase step length to the outer bound
                d_vec=(min(.1*nd+.9*max_d,100*nd))*d_vec/nd;
                min_d=nd;
                accept =1;
            elseif((f_vec(iter+1) <= max(f_vec(max(1,iter-p):iter)))&&(DDfnew>=bDDf))
                % step too long, but not by too much:  decrease step length by one half the
                %                                      interval to the inner bound
                d_vec=(nd+min_d)/2*(d_vec/nd);
                max_d=nd;
                tr=max_d;
            elseif(f_vec(iter+1) >= f_vec(iter))
                % step too long, by a lot:  decrease step length by one half the current steplength
                max_d=nd;
                d_vec=0.5*(min_d+nd)*(d_vec/nd);
                tr=max_d;
                cDDf = gamma*cDDf;
            end
        end
        lower_step_bound = min_d;
        stepsize_vec(iter)  = feval('norm',d_vec);
    end
    %-----------------------------------------
    % end trust region adjust
    %-----------------------------------------
end

% save('mat_vars.mat', 'tr', 'mu_vec', 'cDDf', 'lower_step_bound', ...
%      'd_vec', 'stepsize_vec', 'accept')


clear S_mat Y_mat x_mat gradf_mat delta alpha eta1 eta2 eta3 bDDf gamma iter maxmem
clear History Hessian_product
return
