%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                  sample_dvr.m
%                   Russell Luke
%  Department of MathSci, University of Delaware  
%                        and 
%  Inst. f. Num. u. Angew. Math., Universitaet Goettingen
%                    JUNE. 2, 2010  
%
%  This work was supported by NSF-DMS grant 0712796. 
%
%
% DESCRIPTION:  driver template for the SAMSARA reverse communication 
%               optimization toolbox.
%
% INPUT:  none...this is a script
%
% OUTPUT:
%       xnew_vec,  optimal solution
%       fnew, optimal value
%       diagnost_struct, data structure of diagnostic details, e.g. f_vec,
%                     ngrad_vec, stepsize_vec, iter and x_mat.
%
% USAGE: 
%        sample_dvr
% 
% SUBROUTINE CALLS:
%      SAMSARA
%      Objective
%

clear;
% add samsara toolbox to directory path
addpath('../../');
% clear out old data files
!rm *.dat

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function to minimize
%%%%%%%%%%%%%%%%%%%%%%%%%%%
Objective   = 'Rosenbrock1000';  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
x0_vec=0.09*ones(1000,1);
% [0.09;0.09*ones(499,1);0.09;0.09*ones(499,1)];
dim           = length(x0_vec);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Tolerances:  note that there is a 
%     samsara toerlances OPTIONS_tolerances
%     file that might not match the tolerances
%     you specify here.  Copy OPTIONS_tolernaces
%     into the local directory where the ``driver"
%     program is and set the tolerances with this
%     input file.  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ngrad_TOL = 1e-11;
step_TOL = 1e-14;
Maxit=300;

%%%%%%%%%%%%%%%%%
% Main loop
%%%%%%%%%%%%%%%%%
iter=1;
xold_vec = x0_vec;
% SAMSARA needs two points:  the current point and a proposed point.  
% On the very first iteration the user can either generate the second 
% proposed point manually, or allow SAMSARA to automatically generate 
% the second point.  SAMSARA generates the second point by taking a scaling 
% of the steepest descent direction.  After the first iteration the current step is
% stored and updated in a file maintained by SAMSARA.  
[fold,gradfold_vec]=feval(Objective,xold_vec);
% fold=[];
% d_vec=-.1*gradfold_vec;
% xnew_vec=xold_vec+d_vec;
% stepsize=feval('norm',d_vec)
% [fnew,gradfnew_vec]=feval(Objective,xnew_vec);
% ngradfnew = sqrt(gradfnew_vec'*gradfnew_vec);
d_vec=[];
xnew_vec=[];
stepsize=999;
fnew=[];
gradfnew_vec = [];
samsara_data=[];
ngradfnew=999;
while ((iter<Maxit)&&(ngradfnew>ngrad_TOL)&&(stepsize>step_TOL))
    [xnew_vec,xold_vec,fold,gradfold_vec, stepsize,samsara_data]=...
        feval('samsara', xold_vec, xnew_vec, fold, fnew, gradfold_vec, gradfnew_vec,samsara_data);
    iter=iter+1;
    % new function evaluation:
    [fnew,gradfnew_vec] = feval(Objective,xnew_vec);
    ngradfnew = sqrt(gradfnew_vec'*gradfnew_vec);
end
if(stepsize<=step_TOL)
    disp('Algorithm stagnated:  stepsize tolerance violated.')
end
%%%%%%%%%%%%%%%%%
% end main loop
%%%%%%%%%%%%%%%%%
xopt_vec=xnew_vec;
fopt=fnew;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% prepare diagnostic data and return
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read in OPTIONS:
OPTIONS_params
OPTIONS_tolerances
OPTIONS_opt_choices
%%%%%%%%%
iter                    = samsara_data.state_vec.iter;
x_mat(:,1:maxmem+1)     = samsara_data.stephist_mat.x_mat;
f_vec                   = samsara_data.state_vec.f_vec;
d_vec                   = samsara_data.stephist_mat.d_vec;
ngradf_vec              = samsara_data.state_vec.ngradf_vec;
stepsize_vec            = samsara_data.state_vec.stepsize_vec;
mu_vec                  = samsara_data.state_vec.mu_vec;
delta                   = samsara_data.state_vec.delta;
alpha                   = samsara_data.state_vec.alpha;
tr                      = samsara_data.state_vec.tr;
mem                     = samsara_data.state_vec.mem;
DDf                     = samsara_data.state_vec.DDf;
cDDf                    = samsara_data.state_vec.cDDf;
bDDf                    = samsara_data.state_vec.bDDf;
lower_step_bound        = samsara_data.state_vec.lower_step_bound;
fcounter                = samsara_data.state_vec.fcounter+1;
trstep                  = samsara_data.state_vec.trstep;
gradf_mat(:,1:maxmem+1) = samsara_data.stephist_mat.gradf_mat;
%%%%%%
% write data to disk
stephist_mat = [x_mat(:,1:maxmem+1),gradf_mat(:,1:maxmem+1),d_vec];
state_vec = [f_vec, ngradf_vec, stepsize_vec, mu_vec, iter, delta, ...
    alpha, tr, mem, DDf, cDDf, bDDf, lower_step_bound, fcounter, trstep, Maxit];

[fid,message]=fopen('stephistory.dat','w');
if (fid<0)
    disp(message)
else
    stephist_mat=fwrite(fid,stephist_mat,'real*8');
    fclose(fid);
end

[fid,message]=fopen('state.dat','w');
if (fid<0)
    disp(message)
else
    state_vec=fwrite(fid,state_vec,'real*8');
    fclose(fid);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Graphics generation for diagnostics
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Rosenbrock_viewer

display(['iterations: ',num2str(iter),'; function evaluations: ',num2str(fcounter),';  optimal value: ' num2str(fopt)])


